@@ -21,8 +21,10 @@
 #include <X11/Xutil.h>
 #include <X11/extensions/XTest.h>
 #include <X11/keysym.h>
-#include <X11/Xft/Xft.h>
 
+#ifdef USE_XFT
+#include <X11/Xft/Xft.h>
+#endif
 #include "structs.h"
 #include "kb.h"
 #include "box.h"
@@ -193,6 +195,7 @@ keyboard* kb_new(Window win, Display *display, int kb_x, int kb_y,
 
   /* --- end xft bits -------------------------- */
 
+  kb->xftdraw = NULL;
 #endif
 
   /* Defaults */
@@ -210,7 +213,7 @@ keyboard* kb_new(Window win, Display *display, int kb_x, int kb_y,
 
   if ((rcfp = fopen(conf_file, "r")) == NULL)
     { 
-      fprintf(stderr, "Cant conf file -  %s\n", conf_file);
+      fprintf(stderr, "xkbd: Cant open conf file: %s\n", conf_file);
       exit(1);
     }
     
@@ -270,7 +273,23 @@ keyboard* kb_new(Window win, Display *display, int kb_x, int kb_y,
 		}
 	      if (strncmp(tmpstr_A, "row", 3) == 0)
 		{
-		  
+		  if (kb->total_layouts == 0)
+		    {
+		      /* 
+			 Minor Kludge :-)
+			 So older configs work we can work with
+			 out a <layout> tag 
+		      */
+		      kb->total_layouts++;
+		      kb->kbd_layouts[kb->total_layouts-1] = box_new();
+		      kb->vbox = kb->kbd_layouts[kb->total_layouts-1];
+		      kb->vbox->act_width  = kb_width;
+		      kb->vbox->act_height = kb_height;
+		      kb->vbox->min_height = 0;
+		      kb->vbox->min_width = 0;
+		      kb->vbox->x = kb_x;
+		      kb->vbox->y = kb_y;
+		    }
 		  context=rowdef;
 		  tmp_box = box_add_box(kb->vbox, box_new());
 		  
@@ -285,7 +304,7 @@ keyboard* kb_new(Window win, Display *display, int kb_x, int kb_y,
 		}
 	      
 	    } else {
-	      fprintf(stderr,"Config file parse failed at line: %i\n",
+	      fprintf(stderr,"Config file parse failed (tag) at line: %i\n",
 		      line_no);
 	      exit(1);
 	    }
@@ -607,8 +626,11 @@ void kb_size(keyboard *kb)
    XFillRectangle(kb->display, kb->backing, 
 		  kb->rev_gc, 0, 0, 
 		  kb->vbox->act_width, kb->vbox->act_height);
-   
-#ifdef USE_XFT		
+
+#ifdef USE_XFT		   
+   if (kb->xftdraw != NULL) XftDrawDestroy(kb->xftdraw);
+
+
    kb->xftdraw = XftDrawCreate(kb->display, (Drawable) kb->backing, 
 			       DefaultVisual(kb->display,
 					     DefaultScreen(kb->display)), 
@@ -932,12 +954,19 @@ void kb_send_keypress(button *b)
 
   if (b->kb->state & KB_STATE_SHIFT || b->kb->state & KB_STATE_CAPS)
   {
-     if (b->kb->state & KB_STATE_CAPS)
-     {
+    if (b->kb->state & KB_STATE_SHIFT && b->kb->state & KB_STATE_CAPS)
+      {
+	if (b->default_ks >= 0x061 && b->default_ks <= 0x07a)
+	  ks = b->default_ks;
+	else 
+	  ks = b->shift_ks;
+      }
+    else if (b->kb->state & KB_STATE_CAPS)
+      {
 	/* Keycodes for XK_a to XK_z, need to add il8n chars ! */
 	if (b->default_ks >= 0x061 && b->default_ks <= 0x07a)
-	   ks = b->shift_ks;
-     } else ks = b->shift_ks;
+	  ks = b->shift_ks;
+      } else ks = b->shift_ks;
   }
   else if (b->kb->state & KB_STATE_MOD)
     ks = b->mod_ks;
@@ -979,7 +1008,7 @@ void kb_send_keypress(button *b)
 	  ( (b->kb->state & KB_STATE_CTRL)  || (slide_flag == KB_STATE_CTRL) ),
 	  ( (b->kb->state & KB_STATE_META)  || (slide_flag == KB_STATE_META) ),
 	  ( (b->kb->state & KB_STATE_ALT)   || (slide_flag == KB_STATE_ALT)  ),
-	  ( (b->kb->state & KB_STATE_SHIFT) || (slide_flag == KB_STATE_SHIFT)  )
+		     0 /* ( (b->kb->state & KB_STATE_SHIFT) || (slide_flag == KB_STATE_SHIFT) ) */
 		  );
 
 }
